/*****************************************************************************/
/*                THE FOLLOWING PROGRAM IS THE SOLE PROPERTY OF              */
/*                               RONALD Q. SMITH                             */
/*             CONTAINING HIS PROPRIETARY CONFIDENTIAL INFORMATION           */
/*                       COPYRIGHT RONALD Q. SMITH 1992                      */
/*****************************************************************************/

/* Global data definitions                                                  */

#include <dos.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <clock.h>

/*
PROGRAM:  CLKDEMO

CLKDEMO is used with CLOCK.SYS to provide a minimum set of control and
display functions for handling both the DOS realtime clock and the calendar
clock.  CLKDEMO is intended only to provide an example of a running program
that uses all of the APIs unique to CLOCK.SYS.

INTERFACE:
        CLKDEMO  [C] [D] [/?]

If CLKDEMO is used with no operands it displays the current CONNECTED status
of CLOCK.SYS and the time as obtained from both the DOS clock and the calendar
clock.

Each of the arguments may appear in any order and as many times as desired
except /?.  When /? is encountered, the usage message is displayed and CLKDEMO
exits.  Each argument is a command and is processed as it is encountered.
Thus you may perform a sequence of functions with a single call.

        CLKDEMO C

Connects the two clocks.  When the clocks are connected, any DOS call to
change the date or time, sets both clocks.

        CLKDEMO D

Disconnects the two clocks.  When the clocks are disconnected, any DOS call
to change the date or time, only sets the DOS clock.  The calendar clock is
unchanged.

CLOCK /? displays the interface.  If /? appears anywhere on the command line,
all other arguments are ignored.
*/

/* Global variables                                                         */

int handle;

int main(int argc, unsigned char *argv[])

{

        unsigned char connected;
        struct tm calendar_tm;
        char *timebuf;
        time_t ltime;
        struct dosdate_t dos_date;
        struct dostime_t dos_time;
        unsigned char *arg;
        int iarg;
        void usage(int status, unsigned char *arg);
        void get_handle(void);

/* Display copyright message                                                */

        printf("CLKDEMO %s copyright 1992 Ronald Q. Smith\n",version);

        get_handle();

/* process command line tokens                                              */

        iarg = 0;
        while(++iarg < argc)
        {
            arg = argv[iarg];
            switch (arg[0])
            {
                default:
                    usage(1, arg);

                case 'c':
                case 'C':
                    connect(handle, 1);
                    break;

                case 'd':
                case 'D':
                    connect(handle, 0);
                    break;

                case '-':
                case '/':
                    usage(arg[1] != '?', arg);

            } /* end switch() */

        } /* end while to process command line tokens */

/* Display status.  Also get here if no operands.                           */

        clock_status(handle, &connected, &dos_date, &dos_time);

        if (connected)
            printf("The DOS and calendar clock are connected\n\n");
        else
            printf("The DOS and calendar clock are disconnected\n\n");

        calendar_tm.tm_sec = dos_time.second;
        calendar_tm.tm_min = dos_time.minute;
        calendar_tm.tm_hour = dos_time.hour;
        calendar_tm.tm_mday = dos_date.day;
        calendar_tm.tm_mon = dos_date.month - 1;
        calendar_tm.tm_year = dos_date.year - 1900;
        calendar_tm.tm_wday = 0;
        timebuf = asctime(&calendar_tm);

        printf("The calendar clock is set to %s",&timebuf[4]);

        time(&ltime);
        timebuf=ctime(&ltime);
        printf("     The DOS clock is set to %s",&timebuf[4]);

        return(0);
}

/*
PROCEDURE: usage

"usage" displays the interface to CLKDEMO.  It is called when the /? option is
used on the command line or when any invalid argument appears on the command
line.  A program exit with a code of status occurs.
*/

void usage(int status, unsigned char *arg)

{
        if (status)
            printf("\nIncorrect argument format: %s\n", arg);

        printf("\nCLKDEMO usage:\n\n");
        printf("          CLKDEMO [C] [D] [/?]\n\n");
        printf("CLKDEMO - displays the connected status and \n");
        printf("          the current DOS and calendar clock times.\n\n");
        printf("CLKDEMO C - Connects the DOS and calendar clocks.\n\n");
        printf("CLKDEMO D - Disconnects the clocks.\n\n");
        printf("CLKDEMO /? - displays this message.\n");
        exit(status);
}

/*
PROCEDURE: get_handle

"get_handle" gets a handle for the CLOCK$ device if we don't already have
one.  If we have one, it simply returns.  If an error occurs getting the
handle, a program exit with a code of 2 is taken.
*/

void get_handle()

{
        handle=open_clock();       /* Get file handle for clock    */
        if (handle == -1)
        {
            printf("\nUnable to open CLOCK$ device.\n");
            exit(2);
        }
        return;
}

/*****************************************************************************/
/*                THE PRECEDING PROGRAM IS THE SOLE PROPERTY OF              */
/*                               RONALD Q. SMITH                             */
/*             CONTAINING HIS PROPRIETARY CONFIDENTIAL INFORMATION           */
/*                       COPYRIGHT RONALD Q. SMITH 1992                      */
/*****************************************************************************/
